-- see https://markkarpov.com/tutorial/megaparsec.html
{-# LANGUAGE ImportQualifiedPost #-}
{-# LANGUAGE OverloadedStrings #-}

module Forth.Parser (parse) where

import Data.Text
import Data.Void (Void)
import Forth.Types.Token
import Text.Megaparsec as MP hiding (Token, parse)
import Text.Megaparsec qualified as MP
import Text.Megaparsec.Char qualified as C
import Text.Megaparsec.Char.Lexer qualified as L
import Prelude hiding (div)

type Parser = Parsec Void Text

lexeme :: Parser a -> Parser a
lexeme = L.lexeme C.space

int :: Parser Int
int = lexeme L.decimal

symbol :: Text -> Parser Text
symbol = L.symbol C.space

item :: Parser Token
item =
  Lit
    <$> int
      <|> Add
    <$ symbol "+"
      <|> Sub
    <$ symbol "-"
      <|> Mul
    <$ symbol "*"
      <|> Div
    <$ symbol "/"
      <|> Pop
    <$ symbol "."

items :: Parser [Token]
items = many item

parseItems :: Text -> Either (ParseErrorBundle Text Void) [Token]
parseItems = MP.parse items ""

parse :: Text -> [Token]
parse t = do
  case parseItems t of
    Left err -> [] -- putStrLn $ errorBundlePretty err
    Right tns -> tns
